
/* Chrysalide - Outil d'analyse de fichiers binaires
 * module.c - intégration du répertoire arch en tant que module
 *
 * Copyright (C) 2010-2012 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "module.h"


#include <assert.h>


#include "instriter.h"
#include "instruction.h"
#include "processor.h"
#include "vmpa.h"
#include "arm/module.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Ajoute le module 'arch' au module Python.                    *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool add_arch_module_to_python_module(PyObject *super)
{
    bool result;                            /* Bilan à retourner           */
    PyObject *module;                       /* Sous-module mis en place    */
    int ret;                                /* Bilan d'un appel            */

    static PyModuleDef py_chrysalide_arch_module = {

        .m_base = PyModuleDef_HEAD_INIT,

        .m_name = "pychrysalide.arch",
        .m_doc = "Python module for Chrysalide.arch",

        .m_size = -1,

    };

    result = false;

    module = PyModule_Create(&py_chrysalide_arch_module);
    if (module == NULL) return false;

    ret = PyState_AddModule(super, &py_chrysalide_arch_module);
    if (ret != 0) goto loading_failed;

    ret = _PyImport_FixupBuiltin(module, "pychrysalide.arch");
    if (ret != 0) goto loading_failed;

    Py_INCREF(module);
    ret = PyModule_AddObject(super, "arch", module);
    if (ret != 0) goto loading_failed;

    result = true;

    result &= register_python_arch_instruction(module);
    result &= register_python_arch_processor(module);
    result &= register_python_instr_iterator(module);
    result &= register_python_vmpa(module);
    result &= register_python_mrange(module);

    result &= add_arch_arm_module_to_python_module(module);

 loading_failed:

    assert(result);

    return result;

}
