
/* Chrysalide - Outil d'analyse de fichiers binaires
 * buffercache.c - équivalent Python du fichier "glibext/gbuffercache.h"
 *
 * Copyright (C) 2012 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "buffercache.h"


#include <pygobject.h>


#include <glibext/gbuffercache.h>


#include "../arch/vmpa.h"
#include "../helpers.h"



#if 0
/* Retrouve une ligne au sein d'un tampon avec une adresse. */
static PyObject *py_code_buffer_find_line_by_addr(PyObject *, PyObject *);



/******************************************************************************
*                                                                             *
*  Paramètres  : self = classe représentant un tampon de code.                *
*                args = arguments fournis à l'appel.                          *
*                                                                             *
*  Description : Retrouve une ligne au sein d'un tampon avec une adresse.     *
*                                                                             *
*  Retour      : Instance de la ligne trouvée.                                *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_code_buffer_find_line_by_addr(PyObject *self, PyObject *args)
{
    PyObject *result;                       /* Trouvailles à retourner     */
    PyObject *py_vmpa;                      /* Localisation version Python */
    int ret;                                /* Bilan de lecture des args.  */
    vmpa2t *addr;                           /* Adresse visée par l'opérat° */
    GBuffercache *buffer;                   /* Version native              */
    GBufferLine *line;                      /* Ligne trouvée               */

    ret = PyArg_ParseTuple(args, "O", &py_vmpa);
    if (!ret) return NULL;

    ret = PyObject_IsInstance(py_vmpa, (PyObject *)get_python_vmpa_type());
    if (!ret) return NULL;

    addr = get_internal_vmpa(py_vmpa);
    if (addr == NULL) return NULL;

    buffer = G_CODE_BUFFER(pygobject_get(self));

    line = g_code_buffer_find_line_by_addr(buffer, addr, 0, NULL);
    if (line == NULL) Py_RETURN_NONE;

    result = pygobject_new(G_OBJECT(line));

    return result;

}
#endif


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_buffer_cache_type(void)
{
    static PyMethodDef py_buffer_cache_methods[] = {
#if 0
        {
            "find_line_by_addr", (PyCFunction)py_buffer_cache_find_line_by_addr,
            METH_VARARGS,
            "find_line_by_addr($self, addr, /)\n--\n\nFind a buffer line with a given address."
        },
#endif
        { NULL }
    };

    static PyGetSetDef py_buffer_cache_getseters[] = {
        { NULL }
    };

    static PyTypeObject py_buffer_cache_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.glibext.Buffercache",
        .tp_basicsize   = sizeof(PyGObject),

        .tp_flags       = Py_TPFLAGS_DEFAULT,

        .tp_doc         = "PyChrysalide code buffer",

        .tp_methods     = py_buffer_cache_methods,
        .tp_getset      = py_buffer_cache_getseters,

    };

    return &py_buffer_cache_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.glibext.BufferCache'.  *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool register_python_buffer_cache(PyObject *module)
{
    PyTypeObject *py_buffer_cache_type;     /* Type Python 'BufferCache'   */
    PyObject *dict;                         /* Dictionnaire du module      */

    py_buffer_cache_type = get_python_buffer_cache_type();

    dict = PyModule_GetDict(module);

    if (!register_class_for_pygobject(dict, G_TYPE_BUFFER_CACHE, py_buffer_cache_type, &PyGObject_Type))
        return false;

    return true;

}
